module dados

! objetivo: ler e escrever os dados, alocar memria, 
!           inicializar variveis e calcular dados secundrios

use variaveis

!-------------------------------------------------

implicit none

contains

!-------------------------------------------------

  subroutine le_dados

    dos = system('notepad Stokes_1p3.ent') ! lista dados

    open(7,file='Stokes_1p3.ent')

    read(7,*) caso
    read(7,*) nx
    read(7,*) ny
    read(7,*) dt
    read(7,*) tole
    read(7,*) itmax
    read(7,*) solver
    read(7,*) imax
    read(7,*) tolu
    read(7,*) tolp
    read(7,*) nitm_u
    read(7,*) nitm_p
    read(7,*) w
    read(7,*) Re
    read(7,*) beta
    read(7,*) w_cam
    read(7,*) w_r
    read(7,*) tipo
    read(7,*) ci
    read(7,*) base
    read(7,*) title

    close(7)

! clculo de dados secundrios

    nx = nx + 2
    ny = ny + 2

    L  = 1.0d0
    H  = 1.0d0
    Ut = 1.0d0

    dx = L / (nx-2)
    dy = H / (ny-2)

    nxy = nx * ny

    mi = Ut * L / Re

! alocao de memria

    allocate ( u(nxy),   v(nxy),   p(nxy)  )

    allocate ( uex(nxy), vex(nxy), pex(nxy), bg(nxy)  )

    allocate ( pl(nxy),  ue(nxy),  vn(nxy) )

    allocate ( a(nxy,5), b(nxy),   cup(nxy),  cvp(nxy)  )

    allocate ( ua(nxy),  va(nxy),  uea(nxy),  vna(nxy)  )

    if ( solver == 5 ) allocate ( dl(nxy,4), du(nxy,3) )

! inicializao das variveis

    u   = 0.0d0
    v   = 0.0d0
    p   = 0.0d0

    pl  = 0.0d0
    ue  = 0.0d0
    vn  = 0.0d0
    cup = 0.0d0
    cvp = 0.0d0
    bg  = 0.0d0

    if ( tipo == 1 .or. ci == 1 ) then
       call solucao_exata_nos
    end if

    if ( tipo == 2 ) bg = 0.0d0

    if ( ci == 1 ) then
       call solucao_exata_faces
       u = uex
       v = vex
       p = pex
    end if

    ua  = u
    uea = ue
    va  = v
    vna = vn

  end subroutine le_dados

!-------------------------------------------------

  subroutine mostra_dados

    write(10,1) title, caso, nx-2, ny-2, (nx-2)*(ny-2), dt, tole, itmax,  &
                solver, imax, tolu, tolp, nitm_u, nitm_p, w, L, &
                H, dx, dy, Re, mi, beta, tipo, ci, base
    1 format(/,'ttulo = ',a50,//, &
                  5x,'DADOS',//,  &
                 a20,'   = nome do experimento numrico',//, &
              i8,12x,'   = nmero de volumes reais na direo X',/, &
              i8,12x,'   = nmero de volumes reais na direo Y',/, &
              i8,12x,'   = nmero total de volumes reais',//, &
             1pe22.9,' = incremento de tempo (s)',/, &
             1pe22.9,' = tolerncia sobre as iteraes externas (norma L1 adim.)',/, &
              i8,12x,'   = nmero mximo de iteraes externas',//, &
              i8,12x,'   = solver: 1=Jacobi; 2=Gauss-Seidel; 3=SOR; &
                           4=ADI; 5=MSI',/, &
              i8,12x,'   = nmero mximo de iteraes no ciclo da massa',/, &
             1pe22.9,' = tolerncia no MSI para u,v',/, &
             1pe22.9,' = tolerncia no MSI para plinha',/, &
              i8,12x,'   = nmero mximo de iteraes internas para u,v',/, &
              i8,12x,'   = nmero mximo de iteraes internas para plinha',/, &
             1pe22.9,' = coeficiente de relaxao do SOR',//, &
             1pe22.9,' = comprimento do domnio na direo X',/,  &
             1pe22.9,' = comprimento do domnio na direo Y',/,  &
             1pe22.9,' = comprimento dos volumes na direo X',/,  &
             1pe22.9,' = comprimento dos volumes na direo Y',//,  &
             1pe22.9,' = nmero de Reynolds',/,  &
             1pe22.9,' = viscosidade absoluta (Pa.s)',//,  &
             1pe22.9,' = funo de interpolao: 0=UDS; 1=CDS', //, &
              i8,12x,'   tipo de problema: 1=Shih; 2=Ghia', //, &
              i8,12x,'   tipo da condio inicial: 0=nulo; 1=soluo analtica de Shih', //, &
              i8,12x,'   = malha-base para u e v de interesse')

  end subroutine mostra_dados

!-------------------------------------------------

  subroutine solucao_exata_nos

    real*8 :: x, y ! auxiliares
    real*8 :: fp, flp, gp,  glp, g2lp, g3lp, f3lp ! auxiliares
    real*8 :: fg, f1g, f2g, g1g ! auxiliares
    real*8 :: bcg, ug, vg

! campos

    do i = 1, nx

       x = (i-1.5d+0)*dx
       if ( i == 1  ) x = 0.0d0
       if ( i == nx	) x = L

       do j = 1, ny

          y = (j-1.5d+0)*dy
          if ( j == 1  ) y = 0.0d0
          if ( j == ny ) y = H

          np = (j-1)*nx + i

          fp   = (x**4) - 2.0d0*(x**3) + (x**2)
          flp  = 4.0d0*(x**3) - 6.0d0*(x**2) + 2.0d0*x
          f3lp = 24.0d0*x - 12.0d0
          fg   = 2.0d-1*(x**5) - 5.0d-1*(x**4) + (x**3)/3.0d0
          f1g  = -4.0d0*(x**6) + 12.0d0*(x**5) - 14.0d0*(x**4) &
               +  8.0d0*(x**3) -  2.0d0*(x**2)
          f2g  = 5.0d-1*(fp**2)
          gp   = (y**4) - (y**2)
          glp  = 4.0d0*(y**3) - 2.0d0*y
          g2lp = 12.0d0*(y**2) - 2.0d0
          g3lp = 24.0d0*y
          g1g  = -24.0d0*(y**5) + 8.0d0*(y**3) - 4.0d0*y

          uex(np) = 8.0d0*fp*glp

          vex(np) = -8.0d0*flp*gp

          pex(np) = 8.0d0*mi*(fg*g3lp+flp*glp) &
                  + 64.0d0*f2g*(gp*g2lp-(glp**2))

          bg(np)  = -8.0d0*mi*(24.0d0*fg+2.0d0*flp*g2lp+f3lp*gp) &
                  - 64.0d0*(f2g*g1g-gp*glp*f1g)
 
       end do

    end do

! arquivo ucex.dat para x=L/2

    open(11,file='ucex.dat')

       x = L/2.0d0

       do j = 1, ny

          y = (j-1.5d+0)*dy
          if ( j == 1  ) y = 0.0d0
          if ( j == ny ) y = H

          fp   = (x**4) - 2.0d0*(x**3) + (x**2)

          glp  = 4.0d0*(y**3) - 2.0d0*y

          ug = 8.0d0*fp*glp

          write(11,2) ug, y
 
       end do

    close(11)

    2 format(2(1pe15.6))

! arquivo vcex.dat para y=H/2

    open(11,file='vcex.dat')

    y = H/2.0d0

    do i = 1, nx

       x = (i-1.5d+0)*dx
       if ( i == 1  ) x = 0.0d0
       if ( i == nx	) x = L

          flp  = 4.0d0*(x**3) - 6.0d0*(x**2) + 2.0d0*x

          gp   = (y**4) - (y**2)

          vg = -8.0d0*flp*gp

          write(11,2) x, vg

	end do

    close(11)

  end subroutine solucao_exata_nos

!-------------------------------------------------

  subroutine solucao_exata_faces

    real*8 :: x, y ! auxiliares
    real*8 :: fp, flp, gp, glp ! auxiliares

    do i = 2, nx-2

       x = (i-1)*dx

       do j = 2, ny-1

          y = (j-1.5d+0)*dy

          np = (j-1)*nx + i

          fp   = (x**4) - 2.0d0*(x**3) + (x**2)
          glp  = 4.0d0*(y**3) - 2.0d0*y

          ue(np) = 8.0d0*fp*glp

       end do

    end do

    do i = 2, nx-1

       x = (i-1.5d+0)*dx

       do j = 2, ny-2

          y = (j-1)*dy

          np = (j-1)*nx + i

          flp  = 4.0d0*(x**3) - 6.0d0*(x**2) + 2.0d0*x
          gp   = (y**4) - (y**2)

          vn(np) = -8.0d0*flp*gp

       end do

    end do

  end subroutine solucao_exata_faces

!-------------------------------------------------

  subroutine data_hora

    integer*4    :: var(8) ! data e hora
    character*20 :: vardate,vartime,varzone ! data e hora
    character*2  :: aux1,aux2
    character*4  :: aux3
    character*50 :: aux

    call date_and_time(vardate,vartime,varzone,var)

    write(aux,*) var(3)
    aux1 = trim(adjustl(aux))
    write(aux,*) var(2)
    aux2 = trim(adjustl(aux))
    write(aux,*) var(1)
    aux3 = trim(adjustl(aux))
    dia = '('//trim(aux1)//'/'//trim(aux2)//'/'//aux3//')'

    write(aux,*) var(5)
    aux1 = trim(adjustl(aux))
    write(aux,*) var(6)
    aux2 = trim(adjustl(aux))
    write(aux,*) var(7)
    aux3 = trim(adjustl(aux))
    hora = trim(aux1)//':'//trim(aux2)//':'//aux3

    write(10,18) dia, hora
    18 format(/,5x,'Dia = ',a12,5x,'Hora = ',a8)

    head = trim(title)//" "//trim(dia)//"'"

  end subroutine data_hora

!-------------------------------------------------

end module dados
