module resultados

! objetivo: calcular a soluo numrica e 
!           apresentar grficos e listagens dos resultados

!-------------------------------------------------

use coeficientes
use solvers

implicit none

real*8 fm2, tau, tau1, fp1

contains

!-------------------------------------------------

  subroutine solucao_numerica

    real*8 :: norma_1 ! norma da primeira iterao
    real*8 :: hm      ! tamanho mdio dos volumes de controle

    open(8,file=trim(adjustl(caso))//'.itn')

    write(8,1) 
    1 format(t1,'Iterao',  t12,'fluxo de massa em x=0.5',  t38,'fora',    &
            t62,'u central', t86,'v central',               t110,'p central' )

    open(11,file='r.dat')

    write(10,3) 
    3 format(//,t3,'Iterao',t14,'Norma L1 dos resduos / norma na iterao 1',/)

    tcpu = timef()

    do it = 1, itmax    ! ciclo iterativo externo

       norma = 0.0d0

       ! resolve u

       call coeficientes_velocidade
       call fonte_velocidade_u
       call ccu
       if ( solver == 1 ) call jacobi (nx,ny,nxy,nitm_u,a,b,u)
       if ( solver == 2 ) call gauss_seidel (nx,ny,nxy,nitm_u,a,b,u)
       if ( solver == 3 ) call sor (nx,ny,nxy,nitm_u,w,a,b,u)
       if ( solver == 4 ) call adi (nx,ny,nxy,nitm_u,a,b,u)
       if ( solver == 5 ) then
          call lu2d5(a,dl,du,nxy,nx,ny)
          call fb2d5(a,dl,du,ny,nx,nxy,u,tolu,nitm_u,b)
       end if
       call norma_L1 (nx,ny,nxy,a,b,u,norma)

       ! resolve v

       call fonte_velocidade_v
       call ccv
       if ( solver == 1 ) call jacobi (nx,ny,nxy,nitm_u,a,b,v)
       if ( solver == 2 ) call gauss_seidel (nx,ny,nxy,nitm_u,a,b,v)
       if ( solver == 3 ) call sor (nx,ny,nxy,nitm_u,w,a,b,v)
       if ( solver == 4 ) call adi (nx,ny,nxy,nitm_u,a,b,v)
       if ( solver == 5 ) then
          call lu2d5(a,dl,du,nxy,nx,ny)
          call fb2d5(a,dl,du,ny,nx,nxy,v,tolu,nitm_u,b)
       end if
       call norma_L1 (nx,ny,nxy,a,b,v,norma)

       ! calcula ue, vn

       call media_coef_x_vel ! NHT/94

       do im = 1, imax     ! ciclo iterativo da massa

          ! resolve pl

          call coeficientes_pressao
          call ccp
          if ( solver == 1 ) call jacobi (nx,ny,nxy,nitm_p,a,b,pl)
          if ( solver == 2 ) call gauss_seidel (nx,ny,nxy,nitm_p,a,b,pl)
          if ( solver == 3 ) call sor (nx,ny,nxy,nitm_p,w,a,b,pl)
          if ( solver == 4 ) call adi (nx,ny,nxy,nitm_p,a,b,pl)
          if ( solver == 5 ) then
             call lu2d5(a,dl,du,nxy,nx,ny)
             call fb2d5(a,dl,du,ny,nx,nxy,pl,tolp,nitm_p,b)
          end if
       
          ! correo de p, u, v, ue, vn com pl

          call atualiza_pressao
          call atualiza_velocidades_nodais
          call atualiza_velocidades_medias

       end do

       call norma_L1 (nx,ny,nxy,a,b,pl,norma)

       if ( it == 1 ) norma_1 = norma

       if ( it==1 .or. it==itmax .or. norma/norma_1<=tole .or. mod(it,w_r)==0 ) then
          write(10,4) it, norma/norma_1
          write(11,4) it, norma/norma_1
          4 format(2x,i8,1pe15.6)
          call globais
          write(8,2) it, fm, fp, uc, vc, pc
          2 format(i8,2x,5(1pe24.15))
       end if

       if ( norma/norma_1 <= tole ) exit

       ! atualizao para novo incremento de tempo

       ua  = u
       va  = v
       uea = ue
       vna = vn

    end do

    tcpu = timef()

    write(10,6) norma_1
    6 format(/,5x,'Norma L1 dos resduos na iterao 1 =',1pe15.6)

    close(11)

    call curva_residuo

    deallocate ( a, ua, va, uea, vna, cup, cvp )

    call perfil_u

    call perfil_v

    call chain

    if ( w_cam == 1 ) then

       write(10,10)
       10 format(//,10x,'*** campos ***', //, &
                    10x,'na horizontal: contorno oeste para leste', //, &
                    10x,'na vertical  : contorno sul   para norte'  )

       write(10,11)
       11 format(//,30x,'velocidades u')
       call print1(u,ny,nx,nxy)

       write(10,12)
       12 format(//,30x,'velocidades v')
       call print1(v,ny,nx,nxy)

       write(10,13)
       13 format(//,30x,'velocidades ue')
       call print1(ue,ny,nx,nxy)

       write(10,14)
       14 format(//,30x,'velocidades vn')
       call print1(vn,ny,nx,nxy)

       write(10,15)
       15 format(//,30x,'presses')
       call print1(p,ny,nx,nxy)

       write(10,16)
       16 format(//,30x,'variaes de presso')
       call print1(pl,ny,nx,nxy)

       write(10,17)
       17 format(//,30x,'fontes de plinha')
       call print1(b,ny,nx,nxy)

       write(10,18)
       18 format(//,30x,'funo de corrente')
       call print1(psi,ny,nx,nxy)

    end if

    write(10,19) minval(psi)
    19 format(/,5x,'valor mnimo da funo de corrente = ',1pe24.15)

    call perfil_u_interesse

    call perfil_v_interesse

    hm = dsqrt(1.0d0/((nx-2)*(ny-2)))

    write(10,20) hm, -minval(psi), fp, fp1, uc, vc, pc, tau, tau1, fm2
    20 format(//,5x,'tamanho mdio dos volumes de controle =',1pe24.15,//, &
                 5x,'fluxo de massa total com ue e psi (kg/s) =',1pe24.15,//, &
                 5x,'fora da placa com uP e UDS-2 (N) =',1pe24.15,//, &
                 5x,'fora da placa com uP e UDS (N) =',1pe24.15,//, &
                 5x,'velocidade u central (m/s) =',1pe24.15,//, &
                 5x,'velocidade v central (m/s) =',1pe24.15,//, &
                 5x,'presso central (Pa) =',1pe24.15,//, &
!                 5x,'derivada du/dy com ue e UDS-2 em x=0.5 e y=1 (1/s) =',1pe24.15,//, &
!                 5x,'derivada du/dy com ue e UDS em x=0.5 e y=1 (1/s) =',1pe24.15,//, &
                 5x,'derivada du/dy com uP e UDS-2 em x=0.5 e y=1 (1/s) =',1pe24.15,//, &
                 5x,'derivada du/dy com uP e UDS em x=0.5 e y=1 (1/s) =',1pe24.15,//, &
                 5x,'fluxo de massa com vn em y=0.5 e x[0;0.5] (kg/s) =',1pe24.15)

    write(10,31) tcpu
    31 format(//,f14.2,' = tempo de processamento (s)')

    close(8)

    note_caso = 'notepad '// trim(adjustl(caso)) // '.itn'
    dos = system(note_caso) ! lista arquivo de resultados

  end subroutine solucao_numerica

!-------------------------------------------------

  subroutine norma_L1 (nx,ny,nxy,a,b,x,norma)

    integer,intent(in) :: nx, ny, nxy

    real*8,intent(in),dimension(nxy,5) :: a ! coeficientes
    real*8,intent(in),dimension(nxy)   :: b ! fonte
    real*8,intent(in),dimension(nxy)   :: x ! soluo

    real*8,intent(inout) :: norma

    np = 1 ! canto inferior-esquerdo 
    norma = norma + dabs(x(np)*a(np,3) + a(np,4)*x(np+1) &
          + a(np,5)*x(np+nx) - b(np))

    ! contorno inferior
    j = 1
    do i = 2, nx
       np = (j-1)*nx + i
       norma = norma + dabs(x(np)*a(np,3) &
             + a(np,2)*x(np-1) + a(np,4)*x(np+1) &
             + a(np,5)*x(np+nx) - b(np))
    end do

    ! pontos internos e contornos esquerdo e direito
    do i = 1, nx
       do j = 2, ny-1
          np = (j-1)*nx + i
          norma = norma + dabs(x(np)*a(np,3) + a(np,1)*x(np-nx) &
                + a(np,2)*x(np-1) + a(np,4)*x(np+1) &
                + a(np,5)*x(np+nx) - b(np))
       end do
     end do

    ! contorno superior
    j = ny
    do i = 1, nx-1
       np = (j-1)*nx + i
       norma = norma + dabs(x(np)*a(np,3) + a(np,1)*x(np-nx) &
             + a(np,2)*x(np-1) + a(np,4)*x(np+1) - b(np))
    end do

    ! canto superior-direito
    np = nxy
    norma = norma + dabs(x(np)*a(np,3) + a(np,1)*x(np-nx) &
          + a(np,2)*x(np-1) - b(np))

  end subroutine norma_L1

!-------------------------------------------------

  subroutine globais

    i = nx/2
    j = ny/2
    np = (j-1)*nx + i

    uc = 5.0d-1*(ue(np)+ue(np+nx))

    vc = 5.0d-1*(vn(np)+vn(np+1))

    pc = 2.5d-1*(p(np)+p(np+1)+p(np+nx)+p(np+nx+1))

    i = nx/2
    fm = 0.0d0
    do j = 2, ny-1
       np = (j-1)*nx + i
       if ( ue(np) > 0.0d0 ) fm = fm + ue(np)*dy
    end do

    ! fora da placa com UDS-2
    j = ny-1
    fp = 0.0d0
    do i = 2, nx-1
       np = (j-1)*nx + i
       if ( tipo == 1 ) then
          fp = fp + (8*uex(np+nx) + u(np-nx) - 9*u(np))
       else
          fp = fp + (8*Ut + u(np-nx) - 9*u(np))
       end if
    end do
    fp = fp * mi * dx / ( 3 * dy )

    ! fora da placa com UDS
    j = ny-1
    fp1 = 0.0d0
    do i = 2, nx-1
       np = (j-1)*nx + i
       if ( tipo == 1 ) then
          fp1 = fp1 + (uex(np+nx) - u(np))
       else
          fp1 = fp1 + (Ut - u(np))
       end if
    end do
    fp1 = 2.0d0 * fp1 * mi * dx / dy

    ! derivada du/dy com ue e UDS-2 (tau) e UDS (tau1) em x=0.5 e y=1 (1/s);
    ! vlidos para nx par 
    i  = nx/2
    j  = ny-1
    np = (j-1)*nx + i
!    tau  = (8.0d0 + ue(np-nx) - 9*ue(np))/(3*dy) 
!    tau1 = 2.0d0 * (1.0d0 - ue(np)) / dy

    ! derivada du/dy com uP e UDS-2 (tau) e UDS (tau1) em x=0.5 e y=1 (1/s);
    ! vlidos para nx mpar 
    i  = 1 + nx/2
    j  = ny-1
    np = (j-1)*nx + i
    tau  = (8.0d0 + u(np-nx) - 9*u(np))/(3*dy) 
    tau1 = 2.0d0 * (1.0d0 - u(np)) / dy

    j = ny/2
    fm2 = 0.0d0
    do i = 2, nx/2
       np = (j-1)*nx + i
       fm2 = fm2 + vn(np)*dx
    end do

  end subroutine globais

!-------------------------------------------------

  subroutine print1 ( b, nj, ni, nij )

    integer,intent(in) :: nij, ni, nj
    real*8,intent(in),dimension(nij) :: b ! incgnita a imprimir

    integer :: k, ll, ipart, npi, nps, l

    k=0
    ll=8
    ipart = 1

    2 write(10,11)ipart

    do 1 j=1,nj
       np=(j-1)*ni + k
       npi=np+1
       nps=np+ll
       if(ni.lt.8) nps=np+ni
       write(10,10)(b(l),l=npi,nps)
    1 continue

    if(ni.lt.8) goto 3
    if(nps.eq.nij) goto 3

    k=k+8
    ll=ni-k
    if(ll.gt.8)ll=8
    ipart = ipart + 1

    goto 2

    10 format(8(1pe15.6))
    11 format(/,2x,'parte ',i3)
  
    3 continue

  end subroutine print1

! -----------------------------------------------

  subroutine curva_residuo

    integer :: k ! auxiliar
    real*8 :: uj, y

    open(7,file='r.gnu')

    do k = 1, 6
       read(7,*)
    end do
    write(7,3) head
    3 format("set title '",a62,/, &
             "replot")
    close(7)

    dos = system('wgnuplot r.gnu')

  end subroutine curva_residuo

! -----------------------------------------------

  subroutine perfil_u

    integer :: k ! auxiliar
    real*8 :: uj, y

    open(11,file='u.dat')

    write(10,1)
    1 format(//,5x,'velocidade ue nodal em x = 0.5',//, &
                t4,'ue', t19,'y')

    i = nx/2
    do j = 1, ny
       np = (j-1)*nx + i
       if ( j == 1 ) then
          uj = 0.0d0
          y  = 0.0d0
       else
          uj = ue(np)
          y  = (j-2)*dy + dy/2.0d0
       end if
       if ( j == ny) then
          uj = Ut
          y  = H
       end if
       write(10,2) uj, y
       write(11,2) uj, y
    end do

    close(11)

    if ( tipo == 1 ) then
       open(7,file='u_Shih.gnu')
       do k = 1, 7
          read(7,*)
       end do
    else
       open(7,file='u_Ghia.gnu')
       do k = 1, 9
          read(7,*)
       end do
    end if
    write(7,3) head
    3 format("set title '",a62,/, &
             "replot")

    close(7)

    if ( tipo == 1 ) then
       dos = system('wgnuplot u_Shih.gnu')
    else
       dos = system('wgnuplot u_Ghia.gnu')
    end if

    2 format(2(1pe15.6))

  end subroutine perfil_u

! -----------------------------------------------

  subroutine perfil_v

    integer :: k ! auxiliar
    real*8 :: vi, x

    open(11,file='v.dat')
    1 format(//,5x,'velocidade vn nodal em y = 0.5',//, &
                t4,'x', t19,'vn')

    write(10,1)

    j = ny/2
    do i = 1, nx
       np = (j-1)*nx + i
       if ( i == 1 ) then
          vi = 0.0d0
          x  = 0.0d0
       else
          vi = vn(np)
          x  = (i-2)*dx + dx/2.0d0
       end if
       if ( i == nx) then
          vi = 0.0d0
          x  = L
       end if
       write(10,2) x, vi
       write(11,2) x, vi
    end do

    close(11)

    if ( tipo == 1 ) then
       open(7,file='v_Shih.gnu')
       do k = 1, 6
          read(7,*)
       end do
    else
       open(7,file='v_Ghia.gnu')
       do k = 1, 8
          read(7,*)
       end do
    end if
    write(7,3) head
    3 format("set title '",a62,/, &
             "replot")

    close(7)

    if ( tipo == 1 ) then
       dos = system('wgnuplot v_Shih.gnu')
    else
       dos = system('wgnuplot v_Ghia.gnu')
    end if

    2 format(2(1pe15.6))

  end subroutine perfil_v

! -----------------------------------------------

  subroutine perfil_u_interesse

    integer :: w_int ! auxiliar
    real*8  :: uj, y

    write(10,1)
    1 format(//,5x,'velocidade u de interesse em x = 0.5',//, &
                t4,'u', t28,'y')

    if ( base > (ny-2) ) return

    w_int = (ny-2) / base

    i = nx/2
    do j = 2, ny-w_int
       np = (j-1)*nx + i
       if ( mod((j-1),w_int)==0 ) then
          uj = ( ue(np) + ue(np+nx) ) / 2
          y  = (j-1)*dy
          write(10,2) uj, y
       end if
    end do

    2 format(2(1pe24.15))

  end subroutine perfil_u_interesse

! -----------------------------------------------

  subroutine perfil_v_interesse

    integer :: w_int ! auxiliar
    real*8  :: vi, x

    write(10,1)
    1 format(//,5x,'velocidade v de interesse em y = 0.5',//, &
                t4,'x', t28,'v')

    if ( base > (nx-2) ) return

    w_int = (nx-2) / base

    j = ny/2
    do i = 2, nx-w_int
       np = (j-1)*nx + i
       if ( mod((i-1),w_int)==0 ) then
          vi = ( vn(np) + vn(np+1) ) / 2
          x  = (i-1)*dx
          write(10,2) x, vi
       end if
    end do

    2 format(2(1pe24.15))

  end subroutine perfil_v_interesse

! -----------------------------------------------

  subroutine chain 

    real*8 :: dpsi ! auxiliar

    allocate ( psi(nxy) )

    psi = 0.0d0

    do i = 1, nx-1
       do j = 2, ny-1
          np = (j-1)*nx + i
          dpsi = ue(np) * dy
          psi(np) = psi(np-nx) + dpsi
       end do
    end do

  end subroutine chain 

! -----------------------------------------------

end module resultados
