program STOKES_1p3

!     Escoamento Bidimensional Isotrmico e
!     Permanente de Fluido Incompressvel

!     Verso atual    1.3 ( 6 Out 05)

!     Verso          1.2 (29 Jul 05)
!     Verso          1.1 (30 Abr 99)
!     Verso original 1.0 (25 Mar 99)

!     ltima alterao =   11 Out 05

!     Autor: Carlos Henrique Marchi
!            DEMEC/UFPR, Curitiba, PR


! MODELO MATEMTICO (equaes diferenciais):
! -----------------------------------------

!   massa: du/dx + dv/dy = 0

!   QMLx:  d(u2)/dx + d(uv)/dy = -dp/dx + mi*laplaciano(u)

!   QMLy:  d(uv)/dx + d(v2)/dy = -dp/dy + mi*laplaciano(v) - S

!   x, y = coordenadas espaciais
!   u, v = velocidades nas direes x e y
!   S    = termo fonte; nulo no problema de Ghia
!   Re   = Ut * L / mi (nmero de Reynolds)
!   Ut   = velocidade mxima da tampa da cavidade (1 m/s)
!   Cavidade quadrada (L=1 por H=1 m)
!   ro   = massa especfica (1 kg/m3)
!   Condio de contorno de Dirichlet para u e v
!   Tipos de problemas pr-definidos: 
!     1 = Shih: u na tampa  uma funo de x; u e v nulos nas outras paredes
!     2 = Ghia: u na tampa  constante = 1; u e v nulos nas outras paredes


! MODELO NUMRICO (resumo):
! ------------------------

!   Incgnitas: u, v, p
!   Condies de contorno de Dirichlet para u e v, e p, extrapolado;
!     aplicadas com volumes fictcios
!   Mtodo numrico: volumes finitos
!   Funes de interpolao para u e v: 
!     CDS para os termos difusivos
!     UDS ou CDS (com correo adiada) para os termos advectivos
!   Malha uniforme em cada direo
!   Mtodo de acoplamento presso-velocidade: SIMPLEC
!   Soluo segregada (seqencial) das equaes diferenciais
!   Solvers: Jacobi, Gauss-Seidel, SOR, ADI e MSI-5
!   Preciso: dupla
!   Tempo  usado para avano do processo iterativo
!   Arranjo co-localizado de variveis (mdia NHT/1994)
!   Linguagem FORTRAN 95
!   Aplicativo usado: Compaq Visual Fortran 6.6
!   Tipo de projeto: QuickWin Application
!   Variveis de interesse:
!     Fluxo de massa que escoa na cavidade: obtido com a funo de corrente
!        baseada na velocidade mdia u na face leste de cada volume
!     Fluxo de massa que escoa em y=0.5 e x[0;0.5]: obtido com a integrao 
!        pela regra do retngulo da velocidade mdia v na face norte de cada volume
!     Fora da placa sobre o fluido: obtida com integrao pela regra do retngulo 
!        da derivada du/dy em y=1, obtida com velocidade nodal u e com UDS e UDS-2
!     Derivada du/dy em x=0.5 e y=1: obtida com velocidade nodal u 
!        e com UDS e UDS-2; vlido s para nmero de volumes mpares em x
!     Derivada du/dy em x=0.5 e y=1: obtida com velocidade mdia u 
!        e com UDS e UDS-2; vlido s para nmero de volumes pares em x;
!        clculos desativados
!     Presso no centro do domnio: obtido por mdia aritmtica de p nos
!        quatro ns vizinhos
!     Perfil da velocidade u em x = 0.5: obtido por mdia aritmtica das
!        velocidades na face leste nos dois volumes vizinhos
!     Perfil da velocidade v em y = 0.5: obtido por mdia aritmtica das
!        velocidades na face norte nos dois volumes vizinhos
!   Estimativa inicial da soluo numrica  igual a:
!     Soluo analtica do problema de Shih ou Zero
!   Expresso genrica dos sistemas de equaes discretizados:
!     a(NP,1)*T(NPS) + a(NP,2)*T(NPW) + a(NP,3)*T(NP) + a(NP,4)*T(NPE) 
!                    + a(NP,5)*T(NPN) = b(NP)
!     onde T representa u, v ou plinha


! ARQUIVOS envolvidos no programa:
! -------------------------------

!   Executvel:
!     Stokes_1p3.exe

!   Dados:
!     Stokes_1p3.ent
!     u_Shih.gnu = dados para gerar grfico dos perfis de u no problema de Shih
!     v_Shih.gnu = dados para gerar grfico dos perfis de v no problema de Shih
!     u_Ghia.gnu = dados para gerar grfico dos perfis de u no problema de Ghia
!     v_Ghia.gnu = dados para gerar grfico dos perfis de v no problema de Ghia
!     R.gnu      = dados para gerar grfico dos resduo das equaes ao longo das iteraes
!     Ghia_Re_100_U.dat   = perfil de u no problema de Ghia para Re = 100
!     Ghia_Re_1000_U.dat  = perfil de u no problema de Ghia para Re = 1000
!     Ghia_Re_10000_U.dat = perfil de u no problema de Ghia para Re = 10000
!     Ghia_Re_100_V.dat   = perfil de v no problema de Ghia para Re = 100
!     Ghia_Re_1000_V.dat  = perfil de v no problema de Ghia para Re = 1000
!     Ghia_Re_10000_V.dat = perfil de v no problema de Ghia para Re = 10000

!   Sadas:
!     "caso"     = sada principal do programa
!     "caso".itn = variveis de interesse ao longo das iteraes
!     r.dat      = resduo das equaes ao longo das iteraes
!     ucex.dat   = perfil de u em x=1/2 (exato)
!     vcex.dat   = perfil de v em y=1/2 (exato).
!     u.dat      = perfil de u em x=1/2 (numrico)
!     v.dat      = perfil de v em y=1/2 (numrico)

!   Fontes:
!     Stokes_1p3.f90 = programa principal
!     coef.f90       = calcula coeficientes dos sistemas de equaes
!     dados.f90      = l e lista os dados do programa
!     result.f90     = resolve equaes e gera listagem dos resultados
!     solvers.f90    = solvers disponveis
!     varia.f90      = define todas as variveis globais do programa
!
!   Outros arquivos:
!     Notepad.exe    = editor de arquivos
!     Wgnuplot.exe   = programa gerador de grfico (verso 3.5)

! -----------------------------------------------

use resultados

! -----------------------------------------------

implicit none

!-------------------------------------------------

interface
  logical(4) function initialsettings
  end function
end interface

  name=setwindowtext(gethwndqq(qwin$framewindow),  &
       "STOKES 1.3 (Escoamento Bidimensional de Fluido Incompressvel)"C)

! -----------------------------------------------

  call le_dados

  open(10,file=caso)

  call data_hora

  call mostra_dados

! -----------------------------------------------

  call solucao_numerica

  close (10)

  note_caso = 'notepad '//caso
  dos = system(note_caso) ! lista arquivo de resultados

! -----------------------------------------------

fim = setexitqq(qwin$exitnopersist) 

end

!--------------------------------------------------------

logical(4) function initialsettings()

  use msflib

  record /qwinfo/ qwi
  qwi.x = 100
  qwi.y =  20
  qwi.w = 600         ! 400  600
  qwi.h = 400         ! 200  400
  qwi.type = qwin$set
  i4 = setwsizeqq(qwin$framewindow,qwi)
  qwi.type = qwin$min
  i4 = setwsizeqq(qwin$framewindow,qwi)

  initialsettings = .true.

end function initialsettings

!--------------------------------------------------------
